@@ -1,4 +1,9 @@
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif /* HAVE_CONFIG_H */
+
+#include <glib.h>
+#include <glib/gi18n.h>
 
 #include <unistd.h>
 #include <stdlib.h>
@@ -13,15 +18,11 @@
 #include <sys/ioctl.h>
 #ifdef __FreeBSD__
 #include <sys/uio.h>
-#endif
-#include <glib.h>
+#endif /* __FreeBSD__ */
 #include <signal.h>
-#include <libgnome/gnome-i18n.h>
 
 #include "cd-recorder.h"
 
-#undef DEBUG_CDRECORD
-
 typedef enum {
 	PREPARING_TO_WRITE_CD,
 	WRITING_CD,
@@ -54,6 +55,7 @@ struct CDRecorderPrivate {
 	
 	GList *tracks;
 	int track_count;
+	gboolean debug;
 };
 
 /* Signals */
@@ -75,38 +77,12 @@ static void cd_recorder_init       (CDRecorder      *cdrecorder);
 
 static int cd_recorder_write_cdrecord (CDRecorder *cdrecorder,
 		CDDrive *recorder, GList *tracks,
-		int speed, gboolean eject, gboolean blank,
-		gboolean dummy_write, gboolean disc_at_once);
+		int speed, CDRecorderWriteFlags flags);
 static int cd_recorder_write_growisofs (CDRecorder *cdrecorder,
-		CDDrive *recorder, GList *tracks,
-		int speed, gboolean eject, gboolean blank,
-		gboolean dummy_write);
+		CDDrive *recorder, GList *tracks, int speed,
+		CDRecorderWriteFlags flags);
 
-GType
-cd_recorder_get_type (void)
-{
-	static GType cd_recorder_type = 0;
-
-	if (!cd_recorder_type) {
-		static const GTypeInfo cd_recorder_info = {
-			sizeof (CDRecorderClass),
-			(GBaseInitFunc) NULL,
-			(GBaseFinalizeFunc) NULL,
-			(GClassInitFunc) cd_recorder_class_init,
-			(GClassFinalizeFunc) NULL,
-			NULL /* class_data */,
-			sizeof (CDRecorder),
-			0 /* n_preallocs */,
-			(GInstanceInitFunc) cd_recorder_init,
-		};
-
-		cd_recorder_type = g_type_register_static (G_TYPE_OBJECT,
-				"CDRecorder", &cd_recorder_info,
-				(GTypeFlags)0);
-	}
-
-	return cd_recorder_type;
-}
+G_DEFINE_TYPE(CDRecorder, cd_recorder, G_TYPE_OBJECT)
 
 void
 cd_recorder_track_free (Track *track)
@@ -147,7 +123,7 @@ cd_needs_growisofs (CDDrive *recorder, GList *tracks)
 		return FALSE;
 	}
 
-	type = guess_media_type (recorder->device);
+	type = cd_drive_get_media_type (recorder);
 	if (type == CD_MEDIA_TYPE_DVDR
 			|| type == CD_MEDIA_TYPE_DVDRW
 			|| type == CD_MEDIA_TYPE_DVD_PLUS_R
@@ -209,11 +185,10 @@ growisofs_stdout_read (GIOChannel   *source,
 	status = g_io_channel_read_line (source,
 					 &line, NULL, NULL, NULL);
 
-#ifdef DEBUG_CDRECORD
-	if (line) {
+	if (line && cdrecorder->priv->debug) {
 		g_print ("growisofs stdout: %s", line);
 	}
-#endif
+
 	if (status == G_IO_STATUS_NORMAL) {
 		double percent;
 		int perc_1, perc_2;
@@ -273,11 +248,10 @@ cdrecord_stdout_read (GIOChannel   *source,
 	status = g_io_channel_read_line (source,
 					 &line, NULL, NULL, NULL);
 	
-#ifdef DEBUG_CDRECORD
-	if (line) {
+	if (line && cdrecorder->priv->debug) {
 		g_print ("cdrecord stdout: %s", line);
 	}
-#endif
+
 	if (status == G_IO_STATUS_NORMAL) {
 		if (cdrecorder->priv->line) {
 			g_string_append (cdrecorder->priv->line, line);
@@ -351,11 +325,9 @@ growisofs_stderr_read (GIOChannel   *source,
 
 	status = g_io_channel_read_line (source,
 					 &line, NULL, NULL, NULL);
-#ifdef DEBUG_CDRECORD
-	if (line) {
+	if (line && cdrecorder->priv->debug) {
 		g_print ("growisofs stderr: %s", line);
 	}
-#endif
 
 	/* TODO: Handle errors */
 	if (status == G_IO_STATUS_NORMAL && !cdrecorder->priv->expect_cdrecord_to_die) {
@@ -407,11 +379,10 @@ cdrecord_stderr_read (GIOChannel   *source,
 
 	status = g_io_channel_read_line (source,
 					 &line, NULL, NULL, NULL);
-#ifdef DEBUG_CDRECORD
-	if (line) {
+	if (line && cdrecorder->priv->debug) {
 		g_print ("cdrecord stderr: %s", line);
 	}
-#endif
+
 	/* TODO: Handle errors */
 	if (status == G_IO_STATUS_NORMAL && !cdrecorder->priv->expect_cdrecord_to_die) {
 		g_string_prepend (cdrecorder->priv->cdr_stderr, line);
@@ -420,6 +391,9 @@ cdrecord_stderr_read (GIOChannel   *source,
 				       cd_recorder_table_signals[INSERT_CD_REQUEST], 0,
 				       FALSE);
 			cdrecorder->priv->expect_cdrecord_to_die = TRUE;
+		} else if (strstr (line, "This means that we are checking recorded media.") != NULL) {
+			cdrecorder->priv->last_error = g_strdup (_("The CD has already been recorded"));
+			cdrecorder->priv->result = RESULT_ERROR;
 		} else if (strstr (line, "Cannot blank disk, aborting.") != NULL) {
 			g_signal_emit (G_OBJECT (cdrecorder),
 				       cd_recorder_table_signals[INSERT_RWCD_REQUEST], 0,
@@ -427,8 +401,11 @@ cdrecord_stderr_read (GIOChannel   *source,
 			cdrecorder->priv->expect_cdrecord_to_die = TRUE;
 		} else if (strstr (line, "Data may not fit on current disk") != NULL) {
 			cdrecorder->priv->last_error = g_strdup (_("The files selected did not fit on the CD"));
+			/* FIXME should we error out in that case?
+			cdrecorder->priv->result = RESULT_ERROR; */
 		} else if (strstr (line, "Inappropriate audio coding") != NULL) {
 			cdrecorder->priv->last_error = g_strdup (_("All audio files must be stereo, 16-bit digital audio with 44100Hz samples"));
+			cdrecorder->priv->result = RESULT_ERROR;
 		} else if (strstr (line, "cannot write medium - incompatible format") != NULL) {
 			g_signal_emit (G_OBJECT (cdrecorder),
 				       cd_recorder_table_signals[INSERT_CD_REQUEST], 0,
@@ -454,25 +431,22 @@ cd_recorder_write_tracks (CDRecorder *cdrecorder,
 		       CDDrive *recorder,
 		       GList *tracks,
 		       int speed,
-		       gboolean eject,
-		       gboolean blank,
-		       gboolean dummy_write,
-		       gboolean disc_at_once)
+		       CDRecorderWriteFlags flags)
 		       /* TODO: GError **error */
 {
 	g_return_val_if_fail (tracks != NULL, RESULT_ERROR);
 	cdrecorder->priv->tracks = tracks;
 	cdrecorder->priv->track_count = g_list_length (tracks);
+	cdrecorder->priv->debug = (flags & CDRECORDER_DEBUG);
 
 	if (cd_needs_growisofs (recorder, tracks)) {
 		return cd_recorder_write_growisofs (cdrecorder,
 						    recorder, tracks, speed,
-						    eject, blank, dummy_write);
+						    flags);
 	} else {
 		return cd_recorder_write_cdrecord (cdrecorder,
 						   recorder, tracks, speed,
-						   eject, blank, dummy_write,
-						   disc_at_once);
+						   flags);
 	}
 }
 
@@ -481,9 +455,7 @@ cd_recorder_write_growisofs (CDRecorder *cdrecorder,
 				 CDDrive *recorder,
 				 GList *tracks,
 				 int speed,
-				 gboolean eject,
-				 gboolean blank,
-				 gboolean dummy_write)
+				 CDRecorderWriteFlags flags)
 {
 	GPtrArray *argv;
 	char *speed_str, *dev_str;
@@ -515,7 +487,7 @@ cd_recorder_write_growisofs (CDRecorder *cdrecorder,
 
 	/* Weird, innit? We tell growisofs we have a tty so it ignores
 	 * the fact that the DVD+ has an ISO fs already */
-	if (blank) {
+	if (flags & CDRECORDER_BLANK) {
 		g_ptr_array_add (argv, "-use-the-force-luke=tty");
 	}
 
@@ -549,7 +521,7 @@ retry_growisofs:
 			TRUE);
 	cdrecorder->priv->dangerous = FALSE;
 
-#ifdef DEBUG_CDRECORD
+	if (cdrecorder->priv->debug)
 	{
 		guint i;
 		g_print ("launching command: ");
@@ -558,7 +530,6 @@ retry_growisofs:
 		}
 		g_print ("\n");
 	}
-#endif
 
 	error = NULL;
 	if (!g_spawn_async_with_pipes  (NULL,
@@ -622,7 +593,7 @@ retry_growisofs:
 			cd_recorder_table_signals[ANIMATION_CHANGED], 0,
 			FALSE);
 
-	if (eject != FALSE && cdrecorder->priv->result == RESULT_FINISHED) {
+	if (flags & CDRECORDER_EJECT && cdrecorder->priv->result == RESULT_FINISHED) {
 		char *cmd;
 
 		cmd = g_strdup_printf ("eject %s", recorder->device);
@@ -638,10 +609,7 @@ cd_recorder_write_cdrecord (CDRecorder *cdrecorder,
 		       CDDrive *recorder,
 		       GList *tracks,
 		       int speed,
-		       gboolean eject,
-		       gboolean blank,
-		       gboolean dummy_write,
-		       gboolean disc_at_once)
+		       CDRecorderWriteFlags flags)
 {
 	GPtrArray *argv;
 	char *speed_str, *dev_str;
@@ -662,16 +630,16 @@ cd_recorder_write_cdrecord (CDRecorder *cdrecorder,
 	}
 	dev_str = g_strdup_printf ("dev=%s", recorder->cdrecord_id);
 	g_ptr_array_add (argv, dev_str);
-	if (dummy_write) {
+	if (flags & CDRECORDER_DUMMY_WRITE) {
 		g_ptr_array_add (argv, "-dummy");
 	}
-	if (eject) {
+	if (flags & CDRECORDER_EJECT) {
 		g_ptr_array_add (argv, "-eject");
 	}
-	if (blank) {
+	if (flags & CDRECORDER_BLANK) {
 		g_ptr_array_add (argv, "blank=fast");
 	}
-	if (disc_at_once) {
+	if (flags & CDRECORDER_DISC_AT_ONCE) {
 		g_ptr_array_add (argv, "-dao");
 	}
 	g_ptr_array_add (argv, "-v");
@@ -722,7 +690,7 @@ cd_recorder_write_cdrecord (CDRecorder *cdrecorder,
 		       TRUE);
 	cdrecorder->priv->dangerous = FALSE;
 
-#ifdef DEBUG_CDRECORD
+	if (cdrecorder->priv->debug)
 	{
 		guint i;
 		g_print ("launching command: ");
@@ -731,7 +699,6 @@ cd_recorder_write_cdrecord (CDRecorder *cdrecorder,
 		}
 		g_print ("\n");
 	}
-#endif
 
 	error = NULL;
 	if (!g_spawn_async_with_pipes  (NULL,
