@@ -57,6 +57,7 @@ struct mkisofs_output {
 	int result;
 	int pid;
 	const char *filename;
+	gboolean debug;
 };
 
 static struct mkisofs_output *mkisofs_output_ptr;
@@ -444,11 +445,17 @@ mkisofs_stdout_read (GIOChannel   *source,
 		     GIOCondition  condition,
 		     gpointer      data)
 {
+	struct mkisofs_output *mkisofs_output = data;
 	char *line;
 	GIOStatus status;
 	
 	status = g_io_channel_read_line (source,
 					 &line, NULL, NULL, NULL);
+
+	if (line && mkisofs_output->debug) {
+		g_print ("make_iso stdout: %s", line);
+	}
+
 	if (status == G_IO_STATUS_NORMAL) {
 		g_free (line);
 	}
@@ -465,21 +472,28 @@ mkisofs_stderr_read (GIOChannel   *source,
 	char fraction_str[7];
 	double fraction;
 	GIOStatus status;
-	
+
 	status = g_io_channel_read_line (source,
 					 &line, NULL, NULL, NULL);
 
+	if (line && mkisofs_output->debug) {
+		g_print ("make_iso stderr: %s", line);
+	}
+
 	if (status == G_IO_STATUS_NORMAL) {
-		if (strncmp (line,"Total translation table size", 28) == 0) {
+		if (strncmp (line, "Total translation table size", 28) == 0) {
 			cd_progress_set_fraction (1.0);
 			g_main_loop_quit (mkisofs_output->loop);
 			mkisofs_output->result = RESULT_FINISHED;
 		}
-		
-		if (sscanf (line, "%6c%% done, estimate finish", fraction_str) == 1) {
-			fraction_str[6] = 0;
-			fraction = g_strtod (fraction_str, NULL);
-			cd_progress_set_fraction (fraction/100.0);
+
+		if (strstr (line, "estimate finish")) {
+			if (sscanf (line, "%6c%% done, estimate finish",
+						fraction_str) == 1) {
+				fraction_str[6] = 0;
+				fraction = g_strtod (fraction_str, NULL);
+				cd_progress_set_fraction (fraction/100.0);
+			}
 		}
 
 		g_free (line);
@@ -492,7 +506,7 @@ mkisofs_stderr_read (GIOChannel   *source,
  */
 int
 make_iso (const char *filename, const char *label,
-		gboolean warn_low_space, gboolean use_joliet)
+		gboolean warn_low_space, gboolean use_joliet, gboolean debug)
 {
 	GnomeVFSURI *uri;
 	char *filelist = NULL;
@@ -521,6 +535,8 @@ make_iso (const char *filename, const char *label,
 		g_return_val_if_fail (strlen (label) < 32, RESULT_ERROR);
 	}
 
+	mkisofs_output.debug = debug;
+
 	dirname = g_strdup_printf ("iso-%s.XXXXXX", g_get_user_name ());
 	tempdir = g_build_filename (g_get_tmp_dir (), dirname, NULL);
 	g_free (dirname);
@@ -581,6 +597,15 @@ retry:
 	argv[i++] = NULL;
 
 	error = NULL;
+
+	if (debug) {
+		g_print ("launching command: ");
+		for (i = 0; argv[i] != NULL; i++) {
+			g_print ("%s ", argv[i]);
+		}
+		g_print ("\n");
+	}
+
 	if (!g_spawn_sync (NULL,
 			   (char **)argv,
 			   NULL,
@@ -664,6 +689,14 @@ retry:
 	argv[i++] = filename;
 	argv[i++] = NULL;
 
+	if (debug) {
+		g_print ("launching command: ");
+		for (i = 0; argv[i] != NULL; i++) {
+			g_print ("%s ", argv[i]);
+		}
+		g_print ("\n");
+	}
+
 	cd_progress_set_text (_("Creating CD image"));
 	cd_progress_set_fraction (0.0);
 	cd_progress_set_image_spinning (TRUE);
