@@ -35,6 +35,7 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
+#include <bacon-cd-selection.h>
 
 #include "nautilus-cd-burner.h"
 #include "make-iso.h"
@@ -47,9 +48,10 @@ enum {
 	CANCEL_CD_RECORD,
 };
 
+GtkWidget * target_optionmenu_create (void);
+
 static GladeXML *xml;
 
-static GList *recorders = NULL;
 static CDRecorder *cdrecorder = NULL;
 static int cancel = CANCEL_NONE;
 /* For the image spinning */
@@ -322,13 +324,13 @@ media_type_matches (CDMediaType type, gboolean blank_cd)
 }
 
 static gboolean
-wait_for_insertion (CDDrive *rec, gboolean blank_cd)
+wait_for_insertion (const CDDrive *rec, gboolean blank_cd)
 {
 	CDMediaType type;
 	gboolean reload;
 
 	reload = FALSE;
-	type = guess_media_type (rec->device);
+	type = cd_drive_get_media_type ((CDDrive *)rec);
 
 	if (type == CD_MEDIA_TYPE_ERROR) {
 		reload = TRUE;
@@ -368,7 +370,7 @@ wait_for_insertion (CDDrive *rec, gboolean blank_cd)
 			return FALSE;
 		}
 
-		type = guess_media_type (rec->device);
+		type = cd_drive_get_media_type ((CDDrive *)rec);
 		reload = FALSE;
 		if (type == CD_MEDIA_TYPE_UNKNOWN
 				|| type == CD_MEDIA_TYPE_ERROR) {
@@ -590,8 +592,9 @@ select_iso_filename (void)
 }
 
 static int
-burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject_cd, gboolean blank_cd, int speed, gboolean dummy)
+burn_cd (const CDDrive *rec, const char *source_iso, const char *label, gboolean eject_cd, gboolean blank_cd, int speed, gboolean dummy, gboolean debug)
 {
+	CDRecorderWriteFlags flags;
 	GList *tracks;
 	char *filename, *str;
 	int fd;
@@ -641,7 +644,7 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 			g_object_unref (G_OBJECT (gc));
 			if (path != NULL && path[0] != '\0') {
 				filename = g_build_filename (path, "image.iso.XXXXXX", NULL);
-				fd = mkstemp(filename);
+				fd = g_mkstemp(filename);
 				close (fd);
 			}
 		} else {
@@ -653,7 +656,7 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 		cancel = CANCEL_MAKE_ISO;
 
 		if (filename != NULL) {
-			res = make_iso (filename, label, FALSE, TRUE);
+			res = make_iso (filename, label, FALSE, TRUE, debug);
 		} else {
 			res = RESULT_RETRY;
 		}
@@ -661,17 +664,18 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 		if (res == RESULT_RETRY) {
 			g_free (filename);
 			filename = g_build_filename (g_get_tmp_dir (), "image.iso.XXXXXX", NULL);
-			fd = mkstemp(filename);
+			fd = g_mkstemp(filename);
 			close (fd);
 
-			res = make_iso (filename, label, FALSE, TRUE);
+			res = make_iso (filename, label, FALSE, TRUE, debug);
 			if (res == RESULT_RETRY) {
 				g_free (filename);
 				filename = g_build_filename (g_get_home_dir (), ".image.iso.XXXXXX", NULL);
-				fd = mkstemp(filename);
+				fd = g_mkstemp(filename);
 				close (fd);
 
-				res = make_iso (filename, label, TRUE, TRUE);
+				res = make_iso (filename, label,
+						TRUE, TRUE, debug);
 			}
 
 			if (!res) {
@@ -724,9 +728,22 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 		track->contents.data.filename = g_strdup (filename);
 		tracks = g_list_prepend (tracks, track);
 
-		res = cd_recorder_write_tracks (cdrecorder, rec, tracks, speed, eject_cd, blank_cd, dummy, FALSE);
+		flags = 0;
+		if (eject_cd) {
+			flags |= CDRECORDER_EJECT;
+		}
+		if (blank_cd) {
+			flags |= CDRECORDER_BLANK;
+		}
+		if (dummy) {
+			flags |= CDRECORDER_DUMMY_WRITE;
+		}
+		if (debug) {
+			flags |= CDRECORDER_DEBUG;
+		}
 
-		g_message ("filename: %s", filename);
+		res = cd_recorder_write_tracks (cdrecorder, (CDDrive *)rec,
+				tracks, speed, flags);
 
 		if (source_iso == NULL) {
 			unlink (filename);
@@ -752,17 +769,15 @@ burn_cd (CDDrive *rec, const char *source_iso, const char *label, gboolean eject
 }
 
 
-static CDDrive *
+static const CDDrive *
 lookup_current_recorder (GladeXML *xml)
 {
-	GtkWidget *option_menu;
-	GtkWidget *item;
-	CDDrive *rec;
+	GtkWidget *bcs;
+	const CDDrive *rec;
+
+	bcs = glade_xml_get_widget (xml, "target_optionmenu");
+	rec = bacon_cd_selection_get_cdrom (BACON_CD_SELECTION (bcs));
 
-	option_menu = glade_xml_get_widget (xml, "target_optionmenu");
-	item = gtk_menu_get_active (GTK_MENU (GTK_OPTION_MENU (option_menu)->menu));
-	rec = g_object_get_data (G_OBJECT (item), "cd-drive");
-	
 	return rec;
 }
 
@@ -779,7 +794,7 @@ refresh_dialog (GladeXML *xml)
 	GConfClient *gc;
 	char *name;
 	int i, default_speed;
-	CDDrive *rec;
+	const CDDrive *rec;
 
 	/* Find active recorder: */
 	rec = lookup_current_recorder (xml);
@@ -827,10 +842,8 @@ refresh_dialog (GladeXML *xml)
 }
 
 static void
-target_changed (GtkOptionMenu *option_menu, gpointer data)
+target_changed (BaconCdSelection *bcs, GladeXML *xml)
 {
-  GladeXML *xml = data;
-
   refresh_dialog (xml);
 }
 
@@ -855,15 +868,22 @@ cdname_entry_insert_text (GtkEditable *editable,
 	g_free (current_text);
 }
 
+GtkWidget *
+target_optionmenu_create (void)
+{
+	GtkWidget *widget;
+
+	widget = bacon_cd_selection_new ();
+	g_object_set (widget, "file-image", TRUE, NULL);
+	gtk_widget_show (widget);
+
+	return widget;
+}
 
 static void
 init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc_name, gboolean from_iso)
 {
-	GList *l;
-	CDDrive *rec;
 	GtkWidget *option_menu;
-	GtkWidget *menu;
-	GtkWidget *item;
 	GtkWidget *label;
 	GtkWidget *entry;
 	GtkWidget *reuse;
@@ -879,26 +899,12 @@ init_dialog (GladeXML *xml, GtkWidget *dialog, GnomeVFSFileSize size, char *disc
 				size_mb);
 	gtk_label_set_markup (GTK_LABEL (label), str);
 	g_free (str);
-	
+
 	/* Fill in targets */
 	option_menu = glade_xml_get_widget (xml, "target_optionmenu");
-	menu = gtk_option_menu_get_menu	(GTK_OPTION_MENU (option_menu));
-	for (l = recorders; l != NULL; l = l->next) {
-		rec = l->data;
-		if (rec->type == CDDRIVE_TYPE_FILE) {
-			item = gtk_separator_menu_item_new ();
-			gtk_widget_show (item);
-			gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-		}
-		item = gtk_menu_item_new_with_label (rec->display_name);
-		g_object_set_data (G_OBJECT (item), "cd-drive", rec);
-		gtk_widget_show (item);
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
-	}
-	gtk_option_menu_set_history (GTK_OPTION_MENU (option_menu), 0);
+	g_signal_connect (option_menu, "device-changed",
+			G_CALLBACK (target_changed), xml);
 
-	g_signal_connect (option_menu, "changed", (GCallback)target_changed, xml);
-	
 	/* Set default name */
 	entry = glade_xml_get_widget (xml, "cdname_entry");
 	g_signal_connect (entry, "insert_text", G_CALLBACK (cdname_entry_insert_text),
@@ -1004,7 +1010,7 @@ help_activate (GtkWindow *parent)
 {
 	GError *err = NULL;
 
-	if (gnome_help_display ("user-guide.xml", "gosnautilus-475", &err) == FALSE) {
+	if (gnome_help_display_desktop (NULL, "user-guide", "user-guide.xml", "gosnautilus-475", &err) == FALSE) {
 		char *msg;
 
 		msg = g_strdup_printf (_("There was a problem displaying the help contents: %s."), err->message);
@@ -1094,15 +1100,15 @@ my_gdk_pixbuf_new_from_stock (const char *name)
 int
 main (int argc, char *argv[])
 {
-	CDDrive *rec;
+	const CDDrive *rec;
 	GtkWidget *dialog, *entry, *eject_check, *blank_check, *reuse_check, *progress_window;
 	GtkWidget *cancel_button, *dummy_check, *option_menu, *item;
 	const char *label;
-	int res, speed;
-	gboolean eject, blank, reuse, dummy;
+	int res, speed, i;
+	gboolean eject, blank, reuse, dummy, debug;
 	GnomeVFSURI *uri;
 	GnomeVFSFileSize size;
-	char *source_iso, *disc_name = NULL;
+	char *source_iso, *disc_name;
 	struct stat stat_buf;
 	GConfClient *gc;
 	GdkPixbuf *icon;
@@ -1111,6 +1117,8 @@ main (int argc, char *argv[])
 	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
 	textdomain (GETTEXT_PACKAGE);
 
+	disc_name = NULL;
+
 	gnome_program_init ("nautilus-cd-burner", VERSION,
 			    LIBGNOMEUI_MODULE, argc, argv,
 			    GNOME_PARAM_APP_DATADIR, SHAREDIR,
@@ -1131,7 +1139,13 @@ main (int argc, char *argv[])
 		return 1;
 	}
 
-	source_iso = (argc == 2) ? argv[1] : NULL;
+	source_iso = NULL;
+	for (i = 1; i < argc; i++) {
+		if (strncmp ("--", argv[i], 2) != 0) {
+			source_iso = argv[i];
+			break;
+		}
+	}
 
 	if (source_iso != NULL) {
 		if (!verify_iso (source_iso, &disc_name)) {
@@ -1174,7 +1188,7 @@ main (int argc, char *argv[])
 			return 1;
 		}
 	}
-
+#if 0
 	recorders = scan_for_cdroms (TRUE, source_iso != NULL ? FALSE : TRUE);
 
 	if (recorders == NULL) {
@@ -1190,25 +1204,34 @@ main (int argc, char *argv[])
 		gtk_widget_destroy (dialog);
 		return 1;
 	}
-
+#endif
 	xml = glade_xml_new (DATADIR "/cdburn.glade", NULL, NULL);
 	dialog = glade_xml_get_widget (xml, "cdr_dialog");
 
 	init_dialog (xml, dialog, size, disc_name, source_iso != NULL);
 
-	if (disc_name != NULL) g_free (disc_name);
+	if (disc_name != NULL) {
+		g_free (disc_name);
+	}
 
 	gtk_widget_show (dialog);
 
 	dummy_check = glade_xml_get_widget (xml, "dummy_check");
 
-#ifndef ENABLE_DUMMY_WRITE
-	gtk_widget_hide (dummy_check);
-#endif
+	gc = gconf_client_get_default ();
+	debug = gconf_client_get_bool (gc, "/apps/nautilus-cd-burner/debug", NULL);
+	g_object_unref (G_OBJECT (gc));
+
+	if (debug) {
+		gtk_widget_hide (dummy_check);
+	}
 
 	icon = my_gdk_pixbuf_new_from_stock ("gtk-cdrom");
 	gtk_window_set_icon (GTK_WINDOW (dialog), icon);
+	progress_window = glade_xml_get_widget (xml, "progress_window");
+	gtk_window_set_icon (GTK_WINDOW (progress_window), icon);
 	gdk_pixbuf_unref (icon);
+
 	do {
 		res = gtk_dialog_run (GTK_DIALOG (dialog));
 
@@ -1218,7 +1241,6 @@ main (int argc, char *argv[])
 	} while (res == GTK_RESPONSE_HELP);
 
 	gtk_widget_hide (dialog);
-	progress_window = glade_xml_get_widget (xml, "progress_window");
 	cd_progress_image_setup ();
 	gtk_widget_show (progress_window);
 	cancel_button = glade_xml_get_widget (xml, "cancel_button");
@@ -1239,9 +1261,10 @@ main (int argc, char *argv[])
 
 		dummy = FALSE;
 
-#ifdef ENABLE_DUMMY_WRITE
-		dummy = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dummy_check));
-#endif
+		if (debug) {
+			dummy = gtk_toggle_button_get_active
+				(GTK_TOGGLE_BUTTON (dummy_check));
+		}
 
 		option_menu = glade_xml_get_widget (xml, "speed_optionmenu");
 		item = gtk_menu_get_active (GTK_MENU (gtk_option_menu_get_menu (GTK_OPTION_MENU (option_menu))));
@@ -1250,7 +1273,7 @@ main (int argc, char *argv[])
 		gconf_client_set_int (gc, "/apps/nautilus-cd-burner/default_speed", speed, NULL);
 		g_object_unref (G_OBJECT (gc));
 
-		res = burn_cd (rec, source_iso, label, eject, blank, speed, dummy);
+		res = burn_cd (rec, source_iso, label, eject, blank, speed, dummy, debug);
 
 		reuse_check = glade_xml_get_widget (xml, "reuse_check");
 		reuse = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (reuse_check));
@@ -1272,6 +1295,7 @@ main (int argc, char *argv[])
 	}
 
 	cd_progress_image_cleanup ();
+	gtk_widget_destroy (dialog);
 
 	return 0;
 }
